/**
* @file	C:\Users\Tato\Documents\GitHub\IPOL_MSER\MSERSIFT\src\Descriptor.cpp
*
* @brief	Implements the descriptor class.
*/

#include "Descriptor.h"
#include "Params.h"
#include "AffineNormalizer.h"
#include "GaussianBlur.h"
#include "PATCHSIFT/sift_on_patch.h"

CDescriptor::CDescriptor() : _mser(NULL), _img(NULL)
{
}

/**
* @fn	CTextureDescriptor::Init(CMSER &mser, CGrayscaleImage &img)
*
* @brief	Constructor.
*
* @author	Tato
* @date	22/02/2014
*
* @param [in,out]	mser	The mser.
* @param [in,out]	img 	The image.
*/
void CDescriptor::Init(CMSER &mser, CGrayscaleImage &img) 
{
	_mser = &mser;
	_img = &img;

	_affineInvariantImage = new CGrayscaleImage(CParams::Instance()->Ns, CParams::Instance()->Ns);
	// Get Binary Mask Image
	CGrayscaleImage *input = GetInput(); // mser.GetTextureImageBasedOnMask();

	// Perform normalization
	CAffineNormalizer an;
	CGrayscaleImage *normInput = GetNormalizationInput();
	an.Normalize(mser, *input, *normInput, *_affineInvariantImage, &_rotMatrix, _translation);
	CGaussianBlur gb;
	int sz = (int)ceil(6 * CParams::Instance()->sigmap);
	// Apply blur
	if (sz > 1)
		gb.ApplyOptimizedGaussianBlur(*_affineInvariantImage, _affineInvariantImage, sz, CParams::Instance()->sigmap);

	//delete normInput;
	delete input;

}

/**
* @fn	CTextureDescriptor::~CTextureDescriptor(void)
*
* @brief	Destructor.
*
* @author	Tato
* @date	22/02/2014
*/

CDescriptor::~CDescriptor(void)
{
	delete _affineInvariantImage;
	delete _rotMatrix;
}

/**
* @fn	void CDescriptor::CalculateSIFTDescriptor(vector<double*> &descs)
*
* @brief	Calculates the sift descriptor.
*
* @author	Tato
* @date	22/02/2014
*
* @param [in,out]	descs	[in,out] If non-null, the descs.
*/

void CDescriptor::CalculateSIFTDescriptor(vector<double*> &descs)
{
	// Calculate SIFT descriptor
	CalculateDescriptorsOnPatch(descs, _affineInvariantImage->GetDoubleImage(), _affineInvariantImage->Width(), _affineInvariantImage->Height(), CParams::Instance()->sigmah, CParams::Instance()->sigmad, CParams::Instance()->virtualPatchSize, CParams::Instance()->Nb);
}
