#pragma once
#include "GrayscaleImage.h"

/**
 * @class	CGaussianBlur
 *
 * @brief	A gaussian blur.
 *
 * @author	Tato
 * @date	23/02/2014
 */

class CGaussianBlur
{
public:
	CGaussianBlur(void);
	~CGaussianBlur(void);

	/**
	 * @fn	void CGaussianBlur::ApplyOptimizedGaussianBlur(CGrayscaleImage &img, CGrayscaleImage *res, int tap, double sigma);
	 *
	 * @brief	Applies the optimized gaussian blur.
	 *
	 * @author	Tato
	 * @date	23/02/2014
	 *
	 * @param [in,out]	img	The image.
	 * @param [in,out]	res	If non-null, the resource.
	 * @param	tap		   	The tap.
	 * @param	sigma	   	The sigma.
	 */

	void ApplyOptimizedGaussianBlur(CGrayscaleImage &img, CGrayscaleImage *res, int tap, double sigma);

	/**
	 * @fn	void CGaussianBlur::ApplyGaussianBlur(CGrayscaleImage &img, CGrayscaleImage *res, int tap, double sigma);
	 *
	 * @brief	Applies the gaussian blur.
	 *
	 * @author	Tato
	 * @date	23/02/2014
	 *
	 * @param [in,out]	img	The image.
	 * @param [in,out]	res	If non-null, the resource.
	 * @param	tap		   	The tap.
	 * @param	sigma	   	The sigma.
	 */

	void ApplyGaussianBlur(CGrayscaleImage &img, CGrayscaleImage *res, int tap, double sigma);

};

