//
//  tree.h
//  MSER
//
//  Created by Edouard Oyallon on 9/6/12.
//
//

#ifndef MSER_tree_h
#define MSER_tree_h
#include <set>



/**
 * PixelSet is dedicated to be a fast indexation of the pixel of a connex composant
 * It's used by all algorithms.
 */
struct pixelSet
{
	/// The pointer to the data...(on an array...)
	int *data;
	/// The size of the data
	int size;
};




/// First appeal to node class
class node;

/**
 * The class of the list of nodes
 * This will be a simple list, with a current node bound to others. Usual graph theory is enough to understand how this works.
 * It's in the form \f$(n_4, n_3, n_2, n_1, \emptyset)\f$
 */
class list_node{
public:
	
	/// This contains the current node for a the list.
	node* current_node ;
	/// We have the next node, always for the list
	list_node* next_node;
	/**
	 * If b is the current list node, it will return \f$(a,b)\f$.
	 * In this part, I'm pretty sure there is a best way to do it, please notice me by mail if you have a more elegant solution.
	 * For example, using a constructor ? I don't know.
	 * @param a The used node.
	 */
	list_node* add_node(node* a);
	/**
	 * If b is the current list node, it will return \f$(n,b)\f$ so that n is the node containing the pixel i
	 * @param i The number of the pixel.
	 */
	list_node* add_node(int i, vector<node*> &nodes);
	
	
	/// Constructor of (a,empty)
	list_node(node* a);
	
	/// Constructor of (pixel of intensity i,empty)
	list_node(int i, vector<node*> &nodes);

	~list_node();

};



/**
 * The class of a node of the treee.
 * You have the father, and all the sons(list_node)
 */
class node{
	public :
	
	/// A node contains a list of sons.
	list_node* sons;
	
	/// The list of pixel!
	pixelSet list;
	
    
	/// The ratio is the Aire(I+delta)-Aire(I-delta)/2Delta thing. To find MSER.
	float ratio;
	/// Intensity is just the intensity I of the set of pixels.
	int intensity;
	/// Area of the node.
	int area;
	
	/// Pointer to the parent node.
	node* parent;
	
	/**
	 * Set the father.
	 * @param n The node who is the father.
	 */
	void set_parent(node* n)
	{
		this->parent=n;
	}
	
	/**
	 * Add a son...
	 * @param son The son we want to add.
	 */
	void add_son(node* son)
	{
        
		sons=sons->add_node(son);
	}

	~node()
	{
		if (sons != NULL)
		{
		//	cout << "Delete sons" << endl;
			delete sons;
			sons = NULL;
		}
	}
	
	/**
	 * Constructor
	 * @param h Intensity of the node...
	 */
	node(int h)
	{
		sons=NULL;
		list.data=NULL;
		list.size=0;
		intensity=h;
	}
	
};



#endif
