#include "DescriptorMatcher.h"
#include "DissimilarityScore.h"

CDescriptorMatcher::CDescriptorMatcher(std::vector<std::pair<int, double*> > &desc1, std::vector<std::pair<int, double*> > &desc2, const CMSERList *listMSER1, const CMSERList *listMSER2) : _desc1(desc1), _desc2(desc2), _listMSER1(listMSER1), _listMSER2(listMSER2)
{

}


CDescriptorMatcher::~CDescriptorMatcher(void)
{
}

/**
 * @fn	void CDescriptorMatcher::MatchDescriptors(vector<std::pair<CMSER*,CMSER*> > &resMSER, vector<std::pair<int,int> > &resMatches)
 *
 * @brief	Match descriptors.
 *
 * @author	Tato
 * @date	23/02/2014
 *
 * @param [in,out]	resMSER   	[in,out] If non-null, the resource mser.
 * @param [in,out]	resMatches	The resource matches.
 */

void CDescriptorMatcher::MatchDescriptors(vector<std::pair<CMSER*,CMSER*> > &resMSER, vector<std::pair<int,int> > &resMatches, double matchThreshold)
{
	// Now try to match shape descriptors
	CDissimilarityScore ds;
	//vector<std::pair<CMSER*,CMSER*>> shapeMatches = vector<std::pair<CMSER*,CMSER*>>(_desc1.size());
	//std::vector<std::pair<int,int>> mserShapeMatched = std::vector<std::pair<int,int>>(_desc2.size());
	int qty = 0;
	//#pragma omp parallel for ordered schedule(dynamic) shared(_desc1, _desc2, qty) 
	for (unsigned int i=0;i<_desc1.size();i++)
	{
		int bm;
		double score = ds.GetScore(_desc1[i].second, _desc2, bm, _listMSER2);
		if (score < matchThreshold)
		{
			// First check if the match between both MSERs already exists
			bool alreadyExists = false;
			for (int j=0;j<qty;j++)
			{
				//if (resMatches[j].first == _desc1[i].first && resMatches[j].second == bm)
				if (resMatches[j].second == bm)
					alreadyExists = true;
			}
			// If it does not exist, add it
			if (!alreadyExists)
			{
				//cout << "Score of Match: " << _desc1[i].first << "," << bm << ": " << score << endl;
				resMatches[qty] = std::pair<int, int>(_desc1[i].first, bm);
				resMSER[qty++] = std::pair<CMSER*,CMSER*>(_listMSER1->at(_desc1[i].first), _listMSER2->at(bm));

			}
		}
	}
	resMSER.resize(qty);
	resMatches.resize(qty);
}

