#pragma once
#include "../MSERWrapper.h"
#include <vector>

/**
 * @class	CDescriptorMatcher
 *
 * @brief	A descriptor matcher.
 *
 * @author	Tato
 * @date	23/02/2014
 */

class CDescriptorMatcher
{
	vector<std::pair<int, double*> > &_desc1, &_desc2;
	const CMSERList *_listMSER1, *_listMSER2;

public:

	/**
	 * @fn	CDescriptorMatcher::CDescriptorMatcher(vector<std::pair<int, double*> > &desc1, vector<std::pair<int, double*> > &desc2, const CMSERList *listMSER1, const CMSERList *listMSER2);
	 *
	 * @brief	Constructor.
	 *
	 * @author	Tato
	 * @date	23/02/2014
	 *
	 * @param [in,out]	desc1	[in,out] If non-null, the first description.
	 * @param [in,out]	desc2	[in,out] If non-null, the second description.
	 * @param	listMSER1	 	The first list mser.
	 * @param	listMSER2	 	The second list mser.
	 */

	CDescriptorMatcher(vector<std::pair<int, double*> > &desc1, vector<std::pair<int, double*> > &desc2, const CMSERList *listMSER1, const CMSERList *listMSER2);
	~CDescriptorMatcher(void);

	/**
	 * @fn	void CDescriptorMatcher::MatchDescriptors(vector<std::pair<CMSER*,CMSER*> > &resMSER, vector<std::pair<int,int> > &resMatches);
	 *
	 * @brief	Match descriptors.
	 *
	 * @author	Tato
	 * @date	23/02/2014
	 *
	 * @param [in,out]	resMSER   		[in,out] If non-null, the resource mser.
	 * @param [in,out]	resMatches		The resource matches.
	 * @param [in]		matchThreshold  Threshold
	 */

	void MatchDescriptors(vector<std::pair<CMSER*,CMSER*> > &resMSER, vector<std::pair<int,int> > &resMatches, double matchThreshold);
};

