#pragma once
#include <vector>
#include "../MSERWrapper.h"
//#include "MultiscaleMSER.h"

using namespace std;

/**
 * @class	CDissimilarityScore
 *
 * @brief	A dissimilarity score.
 *
 * @author	Tato
 * @date	23/02/2014
 */

class CDissimilarityScore
{
private:

	/**
	 * @fn	double CDissimilarityScore::CompareDescriptors(double d1[128], double d2[128]);
	 *
	 * @brief	Compare descriptors.
	 *
	 * @author	Tato
	 * @date	23/02/2014
	 *
	 * @param	d1	The first double.
	 * @param	d2	The second double.
	 *
	 * @return	A double.
	 */

	double CompareDescriptors(double d1[128], double d2[128]);

public:
	CDissimilarityScore(void);

	/**
	 * @fn	double CDissimilarityScore::GetScore(double descriptor[128], vector<std::pair<int,double*> > &db, int &bestMatch, const CMSERList *dbMSER);
	 *
	 * @brief	Gets a score.
	 *
	 * @author	Tato
	 * @date	23/02/2014
	 *
	 * @param	descriptor		 	The descriptor.
	 * @param [in,out]	db		 	[in,out] If non-null, the database.
	 * @param [in,out]	bestMatch	A match specifying the best.
	 * @param	dbMSER			 	The database mser.
	 *
	 * @return	The score.
	 */

	double GetScore(double descriptor[128], vector<std::pair<int,double*> > &db, int &bestMatch, const CMSERList *dbMSER);
	~CDissimilarityScore(void);
};

