
/*
IPOL SIFT
Copyright (C) 2013, Ives Rey Otero, CMLA ENS Cachan 
<ives.rey-otero@cmla.ens-cachan.fr>


== Patent Warning and Licence =================================================

The SIFT method is patented 

    [3] "Method and apparatus for identifying scale invariant features
      in an image."
        David G. Lowe
        Patent number: 6711293
        Filing date: Mar 6, 2000
        Issue date: Mar 23, 2004
        Application number: 09/519,89
  
 These source codes are made available for the exclusive aim of serving as
 scientific tool to verify the soundness and completeness of the algorithm
 description. Compilation, execution and redistribution of this file may
 violate patents rights in certain countries. The situation being different
 for every country and changing over time, it is your responsibility to
 determine which patent rights restrictions apply to you before you compile,
 use, modify, or redistribute this file. A patent lawyer is qualified to make
 this determination. If and only if they don't conflict with any patent terms,
 you can benefit from the following license terms attached to this file.


This program is free software: you can use, modify and/or
redistribute it under the terms of the simplified BSD
License. You should have received a copy of this license along
this program. If not, see
<http://www.opensource.org/licenses/bsd-license.html>.

*/
/**
 * @file sift_on_patch.c
 * @brief [[MAIN]] The SIFT method 
 *
 * @li basic SIFT transform applied to one image
 * @li verbose SIFT transform 
 *
 * @author Ives Rey Otero <ives.rey-otero@cmla.ens-cachan.fr>
 */
#include "discrete_representation.h"
#include "sift_description.h"
#include "sift_on_patch.h"



void CalculateDescriptorsOnPatch(std::vector<double*> &descs, double *im, size_t w, size_t h, double sigma_ori, double sigma_descr, double virtualWidth, int numberOfbins)
{
    
    /** Computing gradient images */
    double* im_x = (double*)malloc(w*h*sizeof(double));
    double* im_y = (double*)malloc(w*h*sizeof(double));
    compute_gradient(im, im_x, im_y, w, h);


    /** Computing gradient orientation histogram */
    int nbins1 = numberOfbins; 
    double* hist = (double*)malloc(nbins1*sizeof(double));
    double x = (double)h/2.;
    double y = (double)w/2.;
    double s = 1;
    double sig_win = sigma_ori*w/virtualWidth;
    double K = 1/sigma_ori;
    //int n;
    //n = accumulate_orientation_histogram(x, y, s, im_x, im_y, w, h, nbins1, sig_win, K, hist);
    accumulate_orientation_histogram(x, y, s, im_x, im_y, w, h, nbins1, sig_win, K, hist);


    /** Extracting principal orientation */
    int n_ori;
    double* oris = (double*)malloc(nbins1*sizeof(double));
    double threshold = 0.8;
    bool flagsmooth = false; /* if true smooth the orientation histogram */
    n_ori = extract_principal_orientations(hist, nbins1, threshold, flagsmooth, oris);

    /** Description */
    int nhist = 4;
    int nbins2 = 8;
    sig_win = sigma_descr*w/virtualWidth;
    K = 1/sigma_descr;
    bool flagthresh = 1;
    double threshdes = 0.2;
	descs.resize(n_ori);

    for(int o = 0; o < n_ori; o++){
        double ori = oris[o];
        double* descr = new double[nhist*nhist*nbins2];

        /** Extracting descriptor */
        //n = extract_sift_feature_vector(x, y, s, ori, im_x, im_y, w, h,  nhist, nbins2, sig_win, K, descr);
        extract_sift_feature_vector(x, y, s, ori, im_x, im_y, w, h,  nhist, nbins2, sig_win, K, descr);


        /** Thresholding descriptor */
        threshold_and_quantize_sift_descriptor(descr, nhist, nbins2, flagthresh, threshdes);

        /** Printing Orientation and Descriptor on standard output */
        //fprintf(stdout,"%10.4f       ", ori);
        //for(int i = 0; i < nhist*nhist*nbins2; i++){
        //    fprintf(stdout, "%03i  ", (int)descr[i]);
        //}
        //fprintf(stdout,"\n");

        descs[o] = descr;

        //(void)n;
    }
	free(hist);
	free(oris);
	free(im_x);
	free(im_y);
}





