/**
 * @file	C:\Users\Tato\Documents\GitHub\IPOL_MSER\MSERSIFT\src\Params.cpp
 *
 * @brief	Implements the parameters class.
 */

#include "Params.h"

using namespace std;

CParams* CParams::_instance = NULL; 

/**
 * @fn	CParams::CParams()
 *
 * @brief	Default constructor.
 *
 * @author	Tato
 * @date	21/02/2014
 */

CParams::CParams()
{
	matchThreshold = 0.7;
	delta = 15;
	minimumEllipseLength = 5;
	minDiversity = 0.8;
	minimumMSERArea = 30;
	ransacMaxErr = 7;
	virtualPatchSize = 2.0;
	qtyScales = 3;
	SetDefaultShapeParams();
}

/**
 * @fn	void CParams::SetDefaultShapeParams()
 *
 * @brief	Sets default shape parameters.
 *
 * @author	Tato
 * @date	21/02/2014
 */

void CParams::SetDefaultShapeParams()
{
	// Default parameters
	Ns = 41;
	s = 1.2f;
	sigmap = 1.2f;
	Nb = 38;
	sigmah = 0.5f;
	sigmad = 0.4f; 
	
}

/**
 * @fn	void CParams::SetDefaultTextureParams()
 *
 * @brief	Sets default texture parameters.
 *
 * @author	Tato
 * @date	21/02/2014
 */

void CParams::SetDefaultTextureParams()
{
	// Default parameters
	Ns = 41;
	s = 2.5f;
	sigmap = 1.0f;
	Nb = 38;
	sigmah = 0.3f;
	sigmad = 0.9f;
}

/**
 * @fn	CParams *CParams::Instance()
 *
 * @brief	Gets the instance.
 *
 * @author	Tato
 * @date	21/02/2014
 *
 * @return	null if it fails, else a CParams*.
 */

CParams *CParams::Instance()
{
	if (!_instance)
        _instance = new CParams();
	return _instance;
}