/*
  Copyright 2007, 2008 Computer Vision Lab,
  Ecole Polytechnique Federale de Lausanne (EPFL), Switzerland.
  All rights reserved.

  Authors: Julien Pilet (http://cvlab.epfl.ch/~jpilet)

  This file is part of the ferns_demo software.

  ferns_demo is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  ferns_demo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
  PARTICULAR PURPOSE. See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  ferns_demo; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA
*/
#ifndef CMPHOMO_H
#define CMPHOMO_H

void homography_transform(const double a[2], const double H[3][3], double r[2]);

/* computes the homography sending [0,0] , [0,1], [1,1] and [1,0]
 * to x,y,z and w.
 */
void homography_from_4pt(const double *x, const double *y, const double *z, const double *w, double cgret[8]);

/*
 * computes the homography sending a,b,c,d to x,y,z,w
 */
void homography_from_4corresp(
	const double *a, const double *b, const double *c, const double *d,
	const double *x, const double *y, const double *z, const double *w,
	double R[3][3]);

// generic function
// evaluate the error of a datapoint according to a model
// (implementing this function is necessary for each ransac case)
typedef float (ransac_error_evaluation_function)(
		float *model,
		float *datapoint
		);


// generic function
// compute the model defined from a few data points
// (shall return 0 if no model could be computed)
// (implementing this function is necessary for each ransac case)
typedef int (ransac_model_generating_function)(
		float *out_model,  // parameters of the computed model
		float *data       // data points
		);

// generic function
// tell whether a given model is good enough (e.g., not severely distorted)
// (this function is optional, and only serves as an optimization hint)
typedef bool (ransac_model_accepting_function)(
		float *model);

#endif
