/**
 * @file	C:\Users\Tato\Documents\GitHub\IPOL_MSER\MSERSIFT\src\RansacWrapper.cpp
 *
 * @brief	Implements the ransac wrapper class.
 */

#include "RansacWrapper.h"
#include "RANSAC/Ransac.h"
#include <stdlib.h>

CRansacWrapper::CRansacWrapper(float maxErr)
{
	_maxErr = maxErr;
}

void CRansacWrapper::FilterResults(vector<std::pair<CMSER*,CMSER*> > &matches, vector<std::pair<CMSER*,CMSER*> > &results)
{
	if (matches.size() > 4)
	{
		float *data = new float[matches.size() * 4];
		for (unsigned int i=0;i<matches.size();i++)
		{
			std::pair<CMSER*,CMSER*> match = matches[i];
			float x1= (float) (match.first->GetCentroid()->first * pow(2.0, match.first->GetScale()));
			float y1= (float) (match.first->GetCentroid()->second * pow(2.0, match.first->GetScale()));
			float x2= (float) (match.second->GetCentroid()->first * pow(2.0, match.second->GetScale()));
			float y2= (float) (match.second->GetCentroid()->second * pow(2.0, match.second->GetScale()));

			//const CPixel *m1 = match.first->GetCentroid();
			//const CPixel *m2 = match.second->GetCentroid();
			//float x1 = (float) m1->first;
			//float y1 = (float) m1->second;
			//float x2 = (float) m2->first;
			//float y2 = (float) m2->second;
			data[i*4] = x1;
			data[i*4+1] = y1;
			data[i*4+2] = x2;
			data[i*4+3] = y2;
		}
		float *filteredData = NULL;
		int filteredN;
		bool *filterList = NULL;
		//RunHomographyRansac(data, matches.size(), &filteredData, &filteredN, &filterList,_maxErr);
		RunEpipolarRansac(data, (int) matches.size(), &filteredData, &filteredN, &filterList,_maxErr);
		if (filteredN > 0)
		{
			int qty=0;
			for (unsigned int i=0;i<matches.size();i++)
			{
				if (filterList[i])
					results[qty++] = matches[i];
			}
			results.resize(qty);
		}
		else
		{
			results.resize(0);
		}
		delete[] data;
		if (filteredData != NULL) 
			delete filteredData;
		if (filterList != NULL)
			free(filterList);
	}
	else
	{
		int qty=0;
		for (unsigned int i=0;i<matches.size();i++)
		{
			results[qty++] = matches[i];
		}
		results.resize(qty);

	}
}

CRansacWrapper::~CRansacWrapper(void)
{
}
