/**
 * @file	C:\Users\Tato\Documents\GitHub\IPOL_MSER\MSERSIFT\src\TextureImageGenerator.cpp
 *
 * @brief	Implements the texture image generator class.
 */

#include "TextureImageGenerator.h"
#include "Params.h"
#include "GrayscaleImage.h"
#include "IO_PNG/io_png.h"
#include <math.h>
#include <cstdlib>

/**
 * @fn	CTextureImageGenerator::CTextureImageGenerator(int sz, bool freeShapes)
 *
 * @brief	Constructor.
 *
 * @author	Tato
 * @date	22/02/2014
 *
 * @param	sz		  	The size.
 * @param	freeShapes	true to free shapes.
 */

CTextureImageGenerator::CTextureImageGenerator(int sz, bool freeShapes)
{
	_images = new std::vector<unsigned char *>(sz);
	_freeShapes = freeShapes;
}

/**
 * @fn	CTextureImageGenerator::~CTextureImageGenerator(void)
 *
 * @brief	Destructor.
 *
 * @author	Tato
 * @date	22/02/2014
 */

CTextureImageGenerator::~CTextureImageGenerator(void)
{
	if (_freeShapes)
	{
		for (unsigned int i =0;i<_images->size();i++)
		{
			free(_images->at(i));
		}
	}
	delete _images;
}

/**
 * @fn	void CTextureImageGenerator::addTexture(unsigned char *img, int i)
 *
 * @brief	Adds a texture to 'i'.
 *
 * @author	Tato
 * @date	22/02/2014
 *
 * @param [in,out]	img	If non-null, the image.
 * @param	i		   	Zero-based index of the.
 */

void CTextureImageGenerator::addTexture(unsigned char *img, int i)
{
	_images->at(i) = img;
}

/**
 * @fn	void CTextureImageGenerator::GenerateImage(const char *filename)
 *
 * @brief	Generates an image.
 *
 * @author	Tato
 * @date	22/02/2014
 *
 * @param	filename	Filename of the file.
 */

void CTextureImageGenerator::GenerateImage(const char *filename)
{
	unsigned int amountCols = (unsigned int) sqrt((double) _images->size());
	unsigned int i=0, j = 0;
	int currentPosY = 0, currentPosX = 0;
	CGrayscaleImage img = CGrayscaleImage(CParams::Instance()->Ns*amountCols, (((int) (_images->size() / amountCols)) + 1) * CParams::Instance()->Ns);
	// Get Binary Mask Image
	while (j < _images->size())
	{
		i = 0;
		while (i < amountCols && j < _images->size())
		{
			unsigned char *shape = _images->at(j);
			for (int y=0;y<CParams::Instance()->Ns;y++)
			{
				for (int x=0;x<CParams::Instance()->Ns;x++)
				{
					float val = shape[y*CParams::Instance()->Ns + x];
					img.SetPixel(currentPosX+x,currentPosY+y, val);
				}
			}
			currentPosX += CParams::Instance()->Ns;
			i++;
			j++;
		}
		currentPosY += CParams::Instance()->Ns;
		currentPosX = 0;
	}
	unsigned char *aux = img.GetGrayscaleVersion();
	write_png_u8(filename, aux, img.Width(), img.Height(),1);
	free(aux);
}
