/**
 Sample interface
 */
#ifdef _OPENMP
#include <omp.h>
#endif
#include <iostream>
#include "MserPlusSiftMain.h"
#include <cstdlib>
//#include <vld.h> 

using namespace std;

/**
 * @fn	void ShowUsage()
 *
 * @brief	Shows the usage.
 *
 * @author	Tato
 * @date	20/02/2014
 */

void ShowUsage()
{
	cout << "Usage: MSERSIFT file1.png file2.png [delta] [matchThreshold]" << endl;
	cout << "file1.png and file2.png are the name of the input images. Warning: Only PNG format is supported." << endl;
	cout << "delta is the delta parameter of the MSER computation. Optional. Default value: 15" << endl;
	cout << "matchThreshold is the minimum threshold for a match to be accepted. Optional. Default value: 0.7" << endl;
	cout << "Examples:" << endl;
	cout << "MSERSIFT lena.png lenarot.png 15 0.6" << endl;
	cout << "MSERSIFT lena.png lenarot.png" << endl;
}

/**
 * @fn	int main (int argc, char * const argv[])
 *
 * @brief	Main entry-point for this application.
 *
 * @author	Tato
 * @date	20/02/2014
 *
 * @param	argc	Number of command-line arguments.
 * @param	argv	Array of command-line argument strings.
 *
 * @return	Exit-code for the process - 0 for success, else an error code.
 */

int main (int argc, char * const argv[]) 
{
	#ifdef _OPENMP
	int nProcessors=omp_get_max_threads();
	//cout << "Max Threads: " << nProcessors << endl;
	omp_set_num_threads(nProcessors);
	#endif
	if (argc < 3)
	{
		cout << "Inputs file not specified." << endl;
		ShowUsage();
        return EXIT_FAILURE;
	}
	int delta = 15;
	double threshold = 0.7;
	if (argc > 3)
		delta = atoi(argv[3]);
	if (argc > 4)
		threshold = atof(argv[4]);
	if (argc > 5)
	{
		cout << "Too many parameters specified." << endl;
		ShowUsage();
		return EXIT_FAILURE;
	}

	CMserPlusSiftMain mserPlusSift;
	return mserPlusSift.DoMSERPlusSift(argv[1], argv[2], delta, threshold);

	
}


